@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -37,9 +37,10 @@ import javax.jms.TopicSubscriber;
 
 import org.apache.activemq.broker.jmx.BrokerViewMBean;
 import org.apache.activemq.broker.jmx.TopicViewMBean;
+import org.apache.qpid.jms.JmsConnection;
+import org.apache.qpid.jms.policy.JmsDefaultPresettlePolicy;
 import org.apache.qpid.jms.support.AmqpTestSupport;
 import org.apache.qpid.jms.support.Wait;
-import org.junit.Ignore;
 import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -68,6 +69,9 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
         connection.setClientID("DURABLE-AMQP");
         connection.start();
 
+        assertEquals(0, brokerService.getAdminView().getDurableTopicSubscribers().length);
+        assertEquals(0, brokerService.getAdminView().getInactiveDurableTopicSubscribers().length);
+
         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
         assertNotNull(session);
         Topic topic = session.createTopic(name.getMethodName());
@@ -83,6 +87,8 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
 
         assertEquals(0, brokerService.getAdminView().getDurableTopicSubscribers().length);
         assertEquals(1, brokerService.getAdminView().getInactiveDurableTopicSubscribers().length);
+
+        session.unsubscribe(getSubscriptionName());
     }
 
     @Test(timeout = 60000)
@@ -91,6 +97,9 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
         connection.setClientID("DURABLE-AMQP");
         connection.start();
 
+        assertEquals(0, brokerService.getAdminView().getDurableTopicSubscribers().length);
+        assertEquals(0, brokerService.getAdminView().getInactiveDurableTopicSubscribers().length);
+
         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
         assertNotNull(session);
         Topic topic = session.createTopic(name.getMethodName());
@@ -111,6 +120,9 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
         connection.setClientID("DURABLE-AMQP");
         connection.start();
 
+        assertEquals(0, brokerService.getAdminView().getDurableTopicSubscribers().length);
+        assertEquals(0, brokerService.getAdminView().getInactiveDurableTopicSubscribers().length);
+
         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
         assertNotNull(session);
 
@@ -125,38 +137,15 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
         }
     }
 
-    @Test(timeout = 60000)
-    public void testDurableSubscriptionUnsubscribeInUseThrowsJMSEx() throws Exception {
-        connection = createAmqpConnection();
-        connection.setClientID("DURABLE-AMQP");
-        connection.start();
-
-        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
-        assertNotNull(session);
-        Topic topic = session.createTopic(name.getMethodName());
-        MessageConsumer consumer = session.createDurableSubscriber(topic, getSubscriptionName());
-        assertNotNull(consumer);
-
-        BrokerViewMBean broker = getProxyToBroker();
-        assertEquals(1, broker.getDurableTopicSubscribers().length);
-        assertEquals(0, broker.getInactiveDurableTopicSubscribers().length);
-
-        try {
-            session.unsubscribe(getSubscriptionName());
-            fail("Should have thrown a JMSException");
-        } catch (JMSException ex) {
-        }
-
-        assertEquals(1, broker.getDurableTopicSubscribers().length);
-        assertEquals(0, broker.getInactiveDurableTopicSubscribers().length);
-    }
-
     @Test(timeout = 60000)
     public void testDurableSubscriptionUnsubscribeInUseThrowsAndRecovers() throws Exception {
         connection = createAmqpConnection();
         connection.setClientID("DURABLE-AMQP");
         connection.start();
 
+        assertEquals(0, brokerService.getAdminView().getDurableTopicSubscribers().length);
+        assertEquals(0, brokerService.getAdminView().getInactiveDurableTopicSubscribers().length);
+
         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
         assertNotNull(session);
         Topic topic = session.createTopic(name.getMethodName());
@@ -193,6 +182,9 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
         connection.setClientID("DURABLE-AMQP");
         connection.start();
 
+        assertEquals(0, brokerService.getAdminView().getDurableTopicSubscribers().length);
+        assertEquals(0, brokerService.getAdminView().getInactiveDurableTopicSubscribers().length);
+
         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
         assertNotNull(session);
         Topic topic = session.createTopic(name.getMethodName());
@@ -213,6 +205,10 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
 
         assertEquals(1, brokerService.getAdminView().getDurableTopicSubscribers().length);
         assertEquals(0, brokerService.getAdminView().getInactiveDurableTopicSubscribers().length);
+
+        subscriber.close();
+
+        session.unsubscribe(getSubscriptionName());
     }
 
     @Test(timeout = 60000)
@@ -221,6 +217,9 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
         connection.setClientID("DURABLE-AMQP");
         connection.start();
 
+        assertEquals(0, brokerService.getAdminView().getDurableTopicSubscribers().length);
+        assertEquals(0, brokerService.getAdminView().getInactiveDurableTopicSubscribers().length);
+
         final int MSG_COUNT = 5;
 
         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
@@ -261,15 +260,24 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
         });
 
         assertTrue("Only recieved messages: " + messages.getCount(), messages.await(30, TimeUnit.SECONDS));
+
+        subscriber.close();
+
+        session.unsubscribe(getSubscriptionName());
     }
 
-    @Ignore("Fails currently as ActiveMQ doesn't update the recovered subscription")
     @Test
     public void testDurableResubscribeWithNewNoLocalValue() throws Exception {
         connection = createAmqpConnection();
         connection.setClientID("DURABLE-AMQP");
         connection.start();
 
+        JmsConnection jmsConnection = (JmsConnection) connection;
+        ((JmsDefaultPresettlePolicy) jmsConnection.getPresettlePolicy()).setPresettleAll(true);
+
+        assertEquals(0, brokerService.getAdminView().getDurableTopicSubscribers().length);
+        assertEquals(0, brokerService.getAdminView().getInactiveDurableTopicSubscribers().length);
+
         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
         Topic topic = session.createTopic(getDestinationName());
 
@@ -292,7 +300,7 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
 
         // Durable noLocal=true subscription should not receive them
         {
-            Message message = durableSubscriber.receive(2000);
+            Message message = durableSubscriber.receive(250);
             assertNull(message);
         }
 
@@ -308,14 +316,14 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
         assertTrue("Should have no durables.", Wait.waitFor(new Wait.Condition() {
 
             @Override
-            public boolean isSatisified() throws Exception {
+            public boolean isSatisfied() throws Exception {
                 return brokerService.getAdminView().getDurableTopicSubscribers().length == 0;
             }
         }));
         assertTrue("Should have an inactive sub.", Wait.waitFor(new Wait.Condition() {
 
             @Override
-            public boolean isSatisified() throws Exception {
+            public boolean isSatisfied() throws Exception {
                 return brokerService.getAdminView().getInactiveDurableTopicSubscribers().length == 1;
             }
         }));
@@ -331,7 +339,7 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
         // Durable noLocal=false subscription should not receive them as the subscriptions should
         // have been removed and recreated to update the noLocal flag.
         {
-            Message message = durableSubscriber.receive(2000);
+            Message message = durableSubscriber.receive(250);
             assertNull(message);
         }
 
@@ -343,6 +351,10 @@ public class JmsDurableSubscriberTest extends AmqpTestSupport {
             Message message = durableSubscriber.receive(5000);
             assertNotNull("Should get local messages now", message);
         }
+
+        durableSubscriber.close();
+
+        session.unsubscribe(getSubscriptionName());
     }
 
     private void publishToTopic(Session session, Topic destination) throws Exception {
